@@ -17,11 +17,15 @@
  * Copyright © 2014, Tony Crisci and contributors.
  */
 
-#include <stdio.h>
-#include <glib.h>
-#include <strings.h>
 #include "playerctl-common.h"
 
+#include <glib.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <strings.h>
+
+#define PLAYERCTLD_BUS_NAME "org.mpris.MediaPlayer2.playerctld"
+
 gboolean pctl_parse_playback_status(const gchar *status_str, PlayerctlPlaybackStatus *status) {
     if (status_str == NULL) {
         return FALSE;
@@ -82,7 +86,6 @@ gboolean pctl_parse_loop_status(const gchar *status_str, PlayerctlLoopStatus *st
     }
 
     return FALSE;
-
 }
 
 gchar *pctl_print_gvariant(GVariant *value) {
@@ -132,10 +135,9 @@ PlayerctlSource pctl_bus_type_to_source(GBusType bus_type) {
     }
 }
 
-PlayerctlPlayerName *pctl_player_name_new(const gchar *instance,
-                                          PlayerctlSource source) {
+PlayerctlPlayerName *pctl_player_name_new(const gchar *instance, PlayerctlSource source) {
     PlayerctlPlayerName *player_name = g_slice_new(PlayerctlPlayerName);
-    gchar **split = g_strsplit(instance, ".instance", 2);
+    gchar **split = g_strsplit(instance, ".", 2);
     player_name->name = g_strdup(split[0]);
     g_strfreev(split);
     player_name->instance = g_strdup(instance);
@@ -143,17 +145,14 @@ PlayerctlPlayerName *pctl_player_name_new(const gchar *instance,
     return player_name;
 }
 
-
-gint pctl_player_name_compare(PlayerctlPlayerName *name_a,
-                              PlayerctlPlayerName *name_b) {
+gint pctl_player_name_compare(PlayerctlPlayerName *name_a, PlayerctlPlayerName *name_b) {
     if (name_a->source != name_b->source) {
         return 1;
     }
     return g_strcmp0(name_a->instance, name_b->instance);
 }
 
-gint pctl_player_name_instance_compare(PlayerctlPlayerName *name,
-                                       PlayerctlPlayerName *instance) {
+gint pctl_player_name_instance_compare(PlayerctlPlayerName *name, PlayerctlPlayerName *instance) {
     if (name->source != instance->source) {
         return 1;
     }
@@ -161,9 +160,14 @@ gint pctl_player_name_instance_compare(PlayerctlPlayerName *name,
 }
 
 gint pctl_player_name_string_instance_compare(const gchar *name, const gchar *instance) {
+    if (g_strcmp0(name, "%any") == 0 || g_strcmp0(instance, "%any") == 0) {
+        return 0;
+    }
+
     gboolean exact_match = (g_strcmp0(name, instance) == 0);
-    gboolean instance_match = !exact_match && (g_str_has_prefix(instance, name) &&
-            g_str_has_prefix(instance + strlen(name), ".instance"));
+    gboolean instance_match =
+        !exact_match && (g_str_has_prefix(instance, name) && strlen(instance) > strlen(name) &&
+                         g_str_has_prefix(instance + strlen(name), "."));
 
     if (exact_match || instance_match) {
         return 0;
@@ -178,8 +182,7 @@ GList *pctl_player_name_find(GList *list, gchar *player_id, PlayerctlSource sour
         .source = source,
     };
 
-    return g_list_find_custom(list, &player_name,
-                              (GCompareFunc)pctl_player_name_compare);
+    return g_list_find_custom(list, &player_name, (GCompareFunc)pctl_player_name_compare);
 }
 
 GList *pctl_player_name_find_instance(GList *list, gchar *player_id, PlayerctlSource source) {
@@ -188,11 +191,117 @@ GList *pctl_player_name_find_instance(GList *list, gchar *player_id, PlayerctlSo
         .source = source,
     };
 
-    return g_list_find_custom(list, &player_name,
-                              (GCompareFunc)pctl_player_name_instance_compare);
-
+    return g_list_find_custom(list, &player_name, (GCompareFunc)pctl_player_name_instance_compare);
 }
 
 void pctl_player_name_list_destroy(GList *list) {
+    if (list == NULL) {
+        return;
+    }
     g_list_free_full(list, (GDestroyNotify)playerctl_player_name_free);
 }
+
+GList *pctl_list_player_names_on_bus(GBusType bus_type, GError **err) {
+    GError *tmp_error = NULL;
+    GList *players = NULL;
+    gboolean has_playerctld = FALSE;
+
+    GDBusProxy *proxy = g_dbus_proxy_new_for_bus_sync(
+        bus_type, G_DBUS_PROXY_FLAGS_NONE, NULL, "org.freedesktop.DBus", "/org/freedesktop/DBus",
+        "org.freedesktop.DBus", NULL, &tmp_error);
+
+    if (tmp_error != NULL) {
+        if (tmp_error->domain == G_IO_ERROR && tmp_error->code == G_IO_ERROR_NOT_FOUND) {
+            // XXX: This means the dbus socket address is not found which may
+            // mean that the bus is not running or the address was set
+            // incorrectly. I think we can pass through here because it is true
+            // that there are no names on the bus that is supposed to be at
+            // this socket path. But we need a better way of dealing with this case.
+            const gchar *message = "D-Bus socket address not found, unable to list player names";
+            if (bus_type == G_BUS_TYPE_SESSION) {
+                g_warning("%s", message);
+            } else {
+                g_debug("%s", message);
+            }
+
+            g_clear_error(&tmp_error);
+            return NULL;
+        }
+        g_propagate_error(err, tmp_error);
+        return NULL;
+    }
+
+    g_debug("Getting list of player names from D-Bus");
+    GVariant *reply = g_dbus_proxy_call_sync(proxy, "ListNames", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
+                                             NULL, &tmp_error);
+
+    if (tmp_error != NULL) {
+        g_propagate_error(err, tmp_error);
+        g_object_unref(proxy);
+        return NULL;
+    }
+
+    GVariant *reply_child = g_variant_get_child_value(reply, 0);
+    gsize reply_count;
+    const gchar **names = g_variant_get_strv(reply_child, &reply_count);
+
+    // If playerctld is in the names, get the list of players from there
+    // because it will be in order of activity
+    for (gsize i = 0; i < reply_count; i += 1) {
+        if (g_strcmp0(names[i], PLAYERCTLD_BUS_NAME) == 0) {
+            g_debug("%s", "Playerctld is running. Getting names from there.");
+            has_playerctld = TRUE;
+
+            GDBusProxy *playerctld_proxy = g_dbus_proxy_new_for_bus_sync(
+                bus_type, G_DBUS_PROXY_FLAGS_NONE, NULL, PLAYERCTLD_BUS_NAME,
+                "/org/mpris/MediaPlayer2", "com.github.altdesktop.playerctld", NULL, &tmp_error);
+            if (tmp_error != NULL) {
+                g_warning("Could not get player names from playerctld: %s", tmp_error->message);
+                g_clear_error(&tmp_error);
+                g_object_unref(playerctld_proxy);
+                break;
+            }
+
+            GVariant *playerctld_reply =
+                g_dbus_proxy_get_cached_property(playerctld_proxy, "PlayerNames");
+            if (playerctld_reply == NULL) {
+                g_warning(
+                    "%s",
+                    "Could not get player names from playerctld: PlayerNames property not found");
+                g_clear_error(&tmp_error);
+                g_object_unref(playerctld_proxy);
+                break;
+            }
+
+            g_variant_unref(reply);
+            g_free(names);
+
+            reply = playerctld_reply;
+            names = g_variant_get_strv(reply, &reply_count);
+            g_object_unref(playerctld_proxy);
+            has_playerctld = TRUE;
+
+            break;
+        }
+    }
+
+    size_t offset = strlen(MPRIS_PREFIX);
+    for (gsize i = 0; i < reply_count; i += 1) {
+        if (g_str_has_prefix(names[i], MPRIS_PREFIX)) {
+            PlayerctlPlayerName *player_name =
+                pctl_player_name_new(names[i] + offset, pctl_bus_type_to_source(bus_type));
+            players = g_list_append(players, player_name);
+        }
+    }
+
+    if (!has_playerctld) {
+        players = g_list_sort(players, (GCompareFunc)pctl_player_name_compare);
+    }
+
+    g_object_unref(proxy);
+    g_variant_unref(reply);
+    g_variant_unref(reply_child);
+    g_free(names);
+
+    return players;
+}
